/****************************************************************************
*                             SENDRCV.C
*              demonstrates serial communication on a MODULAR-4/486
*	       board using CQ8; operating system MS-DOS
*
*              version     : 1.G.
*	       last change : 15.10.1998
*
*              version     : 1.F.
*	       last change : 26.06.1998 Fehler in init_cq8_parameters beseitigt
*
*              version     : 1.D.
*	       last change : 29.03.1996
*
*              version     : 1.C.
*	       creation    : 07.12.1995
*	       last change : 07.12.1995
*	       author	   : H. Klinkenborg
*
*    (c) by SORCUS Computer GmbH Heidelberg 1995
****************************************************************************/

/****************************************************************************
*  Description: This program sends a string of characters via the serial
*	 interface A of the MODULAR-4/486 board and receives this string
*	 via the serial interface B of the MODULAR-4/486 board. The CQ8-
*      	 programs M7P0500.LIB and M7P0520.LIB have to be in the subdirectory
*	 SORCUS\CQ8, this example has to be in the subdirectory
*	 SORCUS\CQ8\BEISP\C.
*
*  Compiler: Borland C 3.1
*
*  hardware requirements: PC with monitor, MODULAR-4/486 board
*
*  software requirements: This program requires the library ML7MAIN.LIB
*        and the header ML7BIB.H and the include file ML7ERROR.C. The
*	 pathes of M7P0500.LIB and M7P0520.LIB are given by constant
*	 strings M7P0500 and M7P0520 in SENDRCV.H.
* 	 The operating system OSX on the MODULAR-4/486 board should be
*	 ML7-2F.05x or greater
****************************************************************************/

/****************************************************************************
*  history:
*        1.G. 15.10.1998
*
*        1.F. 26.06.1998 Fehler in init_cq8_parameters beseitigt
*
*        1.D. 29.03.1996 08:30; HK
*	       - parameters clock_send and clock_rcv added
*		 both have to be "1"
*		 (necessary if using M-COM-2)
*        1.C. 07.12.1995 13:46; HK
*	       - creation of the file
****************************************************************************/

/****************************************************************************
*  Includes
****************************************************************************/

#include <conio.h>                     /* Include the default C libraries */
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>

#include <ml7bib.h>  		       /* Include the default library for
					  MODULAR-4/486 and the errorhandling
					  routine */
#include <sendrcv.h>		       /* Include the header file for this
					  source */

/****************************************************************************
*  init_cq8
*     This procedure resets the MODULAR-4/486 board and transfers and in-
*     stalls the communication programs M7P0500.LIB and M7P0520.LIB
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for installation of communication
*		     program M7P0520.LIB (0 = interface A, 1 = interface B)
*
*     first:	     flag used for reset of MODULAR-4
*		     = 0: ML7BIB is initalized, MODULAR-4 board is reset
*			  and macros are made interruptible, interrupt
*			  manager program M7P0500.LIB for both serial
*			  interfaces on MODULAR-4 board is transferred and
*		       	  installed
*		     = 1: communication program M7P0520.LIB for channel is
*			  transferred and installed
*
****************************************************************************/

void init_cq8(byte slot_no, byte channel)
{
  ushort task;

  if (first == 0) {
     ml7_bib_startup(LANGUAGE_C);      /* Init library ML7BIB */

     /* Reset of MODULAR-4/486 board and loading OSX */

     ml7_err = ml7_reset(0,ml7_adr,0,0,0,error_handler,os_string,10);

     if (ml7_err != 0) {	       /* Check error */
	printf("Error during reset of MODULAR-4/486\n");
	ml7_exit();                    /* Close ML7BIB */
	exit(0);                       /* Stop program */
     }

     ml7_set_macro_interruptible(TRUE);/* Set macros interruptible */

     /* Transfer and install communication program for serial interfaces
	on the MODULAR-4 board */

     ml7_err = ml7_transfer_and_install(M7P0500,0,IMtask,0x500,
							interrupt_no,0x809,0);

     if (ml7_err != 0) {	       /* Check error */
	printf("Error during transfer and install of interrupt ",
							 "manager program\n");
	ml7_exit();                    /* Close ML7BIB */
	exit(0);                       /* Stop program */
     }

     first = 1;                        /* Set flag to 1 */
  }
  /* Transfer and install communication program for serial interface, which
     is defined in channel */

  task = 0x310 + 0x08 * slot_no + channel;

  ml7_err = ml7_transfer_and_install(M7P0520,0,task,0x520,0,0x808,0);

  if (ml7_err != 0) {	               /* Check error */
     printf("Error during transfer and install of communication program\n");
     ml7_exit();                       /* Close ML7BIB */
     exit(0);                          /* Stop program */
  }
}

/****************************************************************************
*  init_cq8_parameters
*     This procedure sets some parameters of the task M7P0520.LIB
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*                    (0 = interface A, 1 = interface B)
*
*     quartz:        frequency of quartz on MODULAR-4 board, default
*		     7.3728 MHz
*
*     baudrate:	     baud rate, e.g. 9600 baud
*
*     len:	     bits per character, e.g. 8 bits
*
*     start_stop:    start and stop bits, e.g. 1 stop bit
*
*     parity:	     parity, e.g. no parity = 0
*
****************************************************************************/

void init_cq8_parameters(byte slot_no, byte channel, ulong quartz,
			 ulong baudrate, byte len,
			 byte start_stop, byte parity)
{
  ushort task;			       /* Number of communication task */
  ushort out_data_word = 3;	       /* Prepare send or receive and clear
					  buffer */
  ushort in_data_word = 0;             /* No parameters to be expected */

  ushort outsize = 2;		       /* Two bytes to transfer */

  ushort insize = 0;                   /* No parameters to be returned */

  /* Read parameters from task M7P0520 of defined channel into record
     M7P0520_PAR */

  task = 0x310 + 0x08 * slot_no + channel;

  ml7_read_par_block(task,8,sizeof(M7P0520_PAR)-4,&M7P0520_PAR.slot_no);

  /* Set parameters channel, quartz, baudrate, len, start_stop and parity in
     record M7P0520_PAR */

  M7P0520_PAR.slot_no = slot_no;       /* No. of slot, where module is
					  inserted (0 = base board) */
  M7P0520_PAR.channel = channel;       /* No. of serial interface */
  M7P0520_PAR.clock_send = 1;	       /* Clock : baudrate generator */
  M7P0520_PAR.clock_rcv = 1;	       /* Clock : baudrate generator */
  M7P0520_PAR.quartz = quartz;         /* Quartz-frequency */
  M7P0520_PAR.baudrate = baudrate;     /* Baud rate */
  M7P0520_PAR.len = len;	       /* Bits/character */
  M7P0520_PAR.start_stop = start_stop; /* No. of start/stop bits */
  M7P0520_PAR.parity = parity;	       /* Parity */

  /* Write parameters from record M7P0520_PAR to the task M7P0520 of defined
     channel */

  ml7_write_par_block(task,4,sizeof(M7P0520_PAR),&M7P0520_PAR);

  /* Activate task of defined channel */

  ml7_call_proc(task,2);

  /* Install transmit control of serial interface defined in channel */

  ml7_call_func(task,transmit_ctrl_function,outsize,
			    &out_data_word,0,&insize,&in_data_word,&error);

  /* Check, whether an error occurs during function call */

  check_func_error(slot_no,channel,error,transmit_ctrl_function);

  /* Install receive control of serial interface defined in channel */

  ml7_call_func(task,receive_ctrl_function,outsize,
			    &out_data_word,0,&insize,&in_data_word,&error);

  /* Check, whether an error occurs during function call */

  check_func_error(slot_no,channel,error,receive_ctrl_function);
}

/****************************************************************************
*  send_string
*     This function sends a string via the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     s:             string to be send
*
*     return code:   error code from function call
*
****************************************************************************/

byte send_string (byte slot_no, byte channel, char *s)
{
  ushort dummy_insize;                 /* No. of byte to be returned */
  char   *dummy_in_data_str;	       /* Dummy string for returned byte */
  byte   error;			       /* Error byte of function call */
  ushort task;			       /* Number of communication task */

  /* Calculate task number from slot_no and channel */

  task = 0x310 + 0x08 * slot_no + channel;

  /* Send string via serial interface of MODULAR-4/486 given in channel */

  ml7_call_func(task,send_string_function,strlen(s),s,0,&dummy_insize,
						    dummy_in_data_str,&error);

  return(error);       		       /* Return error code of function call */
}

/****************************************************************************
*  rcv_string
*     This function receives a string via the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     s:             string to be received
*
****************************************************************************/

byte rcv_string (byte slot_no, byte channel, char *s)
{
  char   *dummy_out_data_str;           /* Dummy transfer string */
  ushort maxinsize = 255;               /* Maximum number of byte to be
					   received */
  ushort insize;                        /* Number of bytes received */
  byte   error;                         /* Error byte of function call */
  ushort task;                          /* Number of communication task */

  /* Calculate task number from slot_no and channel */

  task = 0x310 + 0x08 * slot_no + channel;

  /* Receive string via serial interface of MODULAR-4/486 defined in channel */

  ml7_call_func(task,receive_string_function,0,dummy_out_data_str,
						  maxinsize,&insize,s,&error);
  s[insize] = 0;		       /* Set string delimiter */
  return(error);                       /* Return error code of function call */
}

/****************************************************************************
*  check_func_error
*     This procedure checks, wether an error occurs during a function call
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:	     channel number (0 = interface A, 1 = interface B)
*
*     error:         error number returned by function
*
*     func:          function number of corresponding task
*
****************************************************************************/

void check_func_error(byte slot_no, byte channel, byte error, ushort func)
{
  byte   err_status;                   /* Status byte of communication
					  program */
  ushort task;			       /* No. of communication task */

  if (error != 0) {                    /* Print error message, if error
					  <> 0 */
    /* Calculate task number from slot_no and channel */

    task = 0x310 + 0x08 * slot_no + channel;

    /* Print error message */
    gotoxy(1,20);
    printf("Error in function %d of task %X: %d",func,task,error);

    /* Read parameter of communication program and print value */

    ml7_read_par_block(task,0,1,&err_status);
    printf("\nErrorbyte (Parameter 1) = %d\n",err_status);
    ml7_exit();                        /* Close ML7BIB */
    exit(0);                           /* Stop program */
  }
}

/****************************************************************************
*  send_rcv
*     This procedure (main routine) sends a string and receives it
*
****************************************************************************/

void sendrcv(void)
{
  clrscr();			       /* Clear screen */

  /* Print title message to screen */

  printf("Example 1 for CQ8 communication    Version 1.G. 15.10.1998 08:20\n");

  /* Initialize output and input string */

  strcpy(out_data_str,"This is a test of CQ8 communication");
  strcpy(in_data_str1,"");
  strcpy(in_data_str2,"");

  init_cq8(M7P0520_PAR.slot_no,0);     /* Initialize channel 0 on modul in
					  predefined slot */
  init_cq8(M7P0520_PAR.slot_no,1);     /* Initialize channel 1 on modul in
					  predefined slot */
  /* Initialize the parameters quartz, baudrate, len, start_stop and parity
     for channel 0 on modul in predefined slot */

  init_cq8_parameters(M7P0520_PAR.slot_no,0,0x737,150,8,1,0);

  /* Initialize the parameters quartz, baudrate, len, start_stop and parity
     for channel 1 on modul in predefined slot */

  init_cq8_parameters(M7P0520_PAR.slot_no,1,0x737,150,8,1,0);

  error = send_string(M7P0520_PAR.slot_no,0, out_data_str);

  /* Check, whether an error occurs during function call */

  check_func_error(M7P0520_PAR.slot_no,0,error,send_string_function);

  /* Print message to screen */

  gotoxy(1,3);printf("The %4d transmitted characters: %s",
					   strlen(out_data_str),out_data_str);

  /* repeat receiving string until all characters are received or
     keypressed */

  do {
     error = rcv_string(M7P0520_PAR.slot_no,1,in_data_str1);

     /* Concat received string with in_data_str2 */

     strcat(in_data_str2,in_data_str1);

     /* Check, whether an error occurs during function call */

     check_func_error(M7P0520_PAR.slot_no,0,error,receive_string_function);

     gotoxy(1,4);
     printf("The %4d received characters   : %s",
					     strlen(in_data_str2),in_data_str2);
  } while ((strlen(in_data_str2) != strlen(out_data_str)) && (!kbhit()));

  ml7_exit();                	       /* Close ML7BIB */
}

main()
{
 sendrcv();			       /* Start main procedure */
 return(0);                            /* End program */
}