(***************************************************************************)
(* Demo-Unit ML6ERROR.PAS in TURBO PASCAL 6.0 fr Verwendung zusammen mit  *)
(* der Multi-COM PC-Bibliothek.                                            *)
(*                                                                         *)
(* Version von ML6ERROR.PAS: 1.A.    Stand: 22.01.1993                     *)
(*                                                                         *)
(* nderungen vorbehalten						   *)
(*									   *)
(* Status: freigegeben							   *)
(*									   *)
(* Die Unit zeigt, wie eine Fehlerbehandlungs-Routine fr die ML6-         *)
(* Bibliothek aufgebaut sein kann. Die wichtigste Prozedur error_handler   *)
(* ist als Muster gedacht, das Sie nach Ihren eigenen Erfordernissen       *)
(* verndern sollten.                                                      *)
(***************************************************************************)

UNIT ML7ERROR;

INTERFACE

USES ML7BIB, CRT;

(* Muster-Fehlerbehandlungs-Prozedur **************************************)
PROCEDURE error_handler;

(* Konvertierungsroutine fr Hexadezimale Ausgabe *************************)
FUNCTION hexF(zahl : LONGINT; stellen : INTEGER) : STRING;
          {wandelt eine Longint-Zahl in einen Hexadezimal-String.
          Der String wird auf (stellen) Stellen formatiert, d.h. fhrende
          Nullen werden erzeugt.}

(* Konvertierungsroutine fr binre Ausgabe *******************************)
FUNCTION BIT(b:byte) : string;
	  {Umwandlung eines Byte in einen String der Lnge 8
	   von Binrzahlen (z.B. 01010101}

TYPE

  ML7_error_info_type = record
  			{Ausschnitt aus cardvars fr Fehlerbearbeitung}
  			{soll identisch bleiben mit cardvars}
               ok             : BOOLEAN;
               repeatmacro    : BOOLEAN;
               errorcode      : BYTE;
               errorclass     : BYTE;
               statusport     : BYTE;
		       statusmap      : BYTE;
				{ Bitmap: nur in Fehlersituationen <> 0 gesetzt
                     0 = Fehler ohne ML7_clear_error
                     1 = ( wie bit 0, jedoch beim Empfangen)
                     2 = frei
                     3 = frei
                     4 = frei
                     5 = frei
                     6 = Error-Request
		     		 7 = Bib fr Karte nicht initial.}
	       errorrequest   : WORD;
	       sndcount	      : WORD;
	       rcvcount	      : WORD;
	       mark	      : WORD;
	       markmessage    : STRING[80];
	       outwords	      : ARRAY [0..5] of WORD;
	       inwords	      : ARRAY [0..5] of WORD;
	       datawords      : ARRAY [0..5] of WORD;
	       errormessage   : STRING[80];
           maxerrorretry  : WORD;
           errorretry	  : WORD;

  end;



IMPLEMENTATION


(**************************************************************************)
(* Dies ist die Muster-Fehlerbehandlungs-Prozedur *************************)
(**************************************************************************)
PROCEDURE error_handler;   {Die Fehlerbehandlungsprozedur mu eine FAR
                            -Prozedur sein. Die ist durch das Unit-Inter-
                            face jedoch schon gewhrleistet }
VAR
    err_rec  	: ML7_error_info_type;
    w       	: WORD;
    i       	: INTEGER;
    ch			: CHAR;
    FCA,FCB 	: BYTE;
    snd_soll,
    rcv_soll 	: WORD;

BEGIN
  ML7_get_error_info(err_rec);


  if err_rec.errorclass=5 then BEGIN
    w:=ML7_get_message;

    writeln('Die Fehlermeldung wurde durch einen Errorrequest');
    writeln('von der MODULAR-4 Karte ausgelst');
    writeln;
    write('Fehler-Request von Karte: ');
    writeln('     SRQ: ',hexf(w, 4));
    writeln('Bitte eine Taste drcken');
    repeat until keypressed;
    while keypressed do ch:=readkey;
    exit;
  END;
  if err_rec.mark=0 then exit;	{Falls ein Fehler in ML7_start oder in
                                 ML7_reset auftritt, soll ML7_start (resp.
                                 ML7_reset) den Fehlercode zurckgeben.}
  writeln(#7);
  writeln;
  if err_rec.statusmap and 128 > 0 then BEGIN
    writeln('Kommunikationsversuch ohne Initialisierung der ');
    writeln('Bibliothek fr die angewhlte Karte');
    writeln('Verwenden Sie ML7_start oder ml4_reset');
    writeln('Programm angehalten');
    halt;
  END;
  if err_rec.statusmap and 1 > 0 then BEGIN
    writeln('Kommunikationsversuch ohne ausreichende Fehlerbehandlung.');
    writeln('Verwenden Sie ML7_clear_error in der Fehlerbehandlungsroutine.');
    writeln('Die Fehlervariablen enthalten noch den ursprnglichen Fehler.');
    writeln;
  END;

  with err_rec do BEGIN
    write  ('Fehlermeldung: ');
    highvideo;
    writeln(errormessage);
    lowvideo;
    writeln('Mark-Meldung:  ',markmessage);
    write  ('Fehler-Code:   ',errorcode, '    Fehlerklasse: ',errorclass);
    write  ('     Marke: ',mark);
    if errorclass in [0, 4] then begin
      writeln('     Status-Port: ',bit(statusport),'B');
    end;
    if statusmap and 64 > 0 then BEGIN
      writeln;
      write('Fehler-Request von Karte: ');
      writeln('     SRQ: ',hexf(ML7_get_message,4));
      writeln;
    END;
    if ((errorclass in [0,4]) and (errorcode<14)) then BEGIN
    		{Detaillierte Fehleranalyse bei Makro-Kommunikationsfehler}
      write  ('Macrobefehl:   ');
      					{Berechnung der Makro-Lnge in Byte}
      FCB := hi(outwords[0]) and $0f;	{Befehlslnge}
      FCA := (hi(outwords[0]) and $f0) shr 4;	{Antwortlnge}
      if FCB=$f then BEGIN	{Blockschreibbefehl}
        snd_soll:=FCB+4+outwords[1];
      END else BEGIN
        if FCA=$f then BEGIN	{Blocklesebefehl}
          snd_soll:=FCB+4;
          rcv_soll:=1+outwords[1];
        END else BEGIN    	{Standard Makro}
          snd_soll:=FCB+2;
          rcv_soll:=FCA;
        END
	  END;
      i:=snd_soll;
	  if snd_soll>12 then i:=12;		{Maximale Ausgabelnge}

      w:=0;                  	{Wort!-Zhler}
      highvideo;
      while i>0 do BEGIN
        if w >= sndcount div 2 then lowvideo;
        write(hexf(outwords[w],4),'h ');
        inc(w); dec(i,2);
      END;
      writeln; lowvideo;
      if rcvcount>0 then BEGIN
        write('Antwort:       ');
        i:=rcvcount;
        w:=0;
        highvideo;
        while i>0 do BEGIN
          write(hexf(inwords[w],4),'h ');
          inc(w); dec(i,2);
        END;
        lowvideo;
        writeln;
      END;
      writeln('gesendet:      ',sndcount,' Byte    (erwartet: ',snd_soll,')');
      writeln('empfangen:     ',rcvcount,' Byte    (erwartet: ',rcv_soll,')');
      writeln;
    END;
  END;
  ML7_clear_error;
  if ML7_error_repeat_test then BEGIN
    ML7_set_repeat_macro(TRUE);
    writeln('Der Makrobefehl wird wiederholt');
    writeln; writeln;
    w:=ML7_clear;	{Bereinigung versuchen}
    if w=0
      then writeln('Die Funktion ML7_clear wurde erfolgreich ausgefhrt.')
      else  writeln('Die Funktion ML7_clear nicht wurde erfolgreich ausgefhrt.');
  END ELSE BEGIN
    writeln('Zugelassene Anzahl Wiederholungen (',err_rec.maxerrorretry,') erreicht.');
    {  ML7_set_error_retry(1);}	{hier wegen halt nicht sinnvoll}
    writeln(' Das Programm wird angehalten.');
    ML7_exit;
    halt;
  END;
END;

(**************************************************************************)
(* Konvertierungsroutinen fr Hexadezimale Ausgabe ************************)
(**************************************************************************)
FUNCTION hex(zahl :LONGINT) : STRING;
  {wandelt eine beliebige LONGINT-Zahl in einen Hexadezimal-String}
VAR s : STRING[8];
    ziffer : Byte;
    w,wx   : WORD;
const
	HEXCONV : array [0..$F] of char = ('0','1','2','3','4','5','6','7',
					   '8','9','A','B','C','D','E','F');
BEGIN
  w:=0;
  REPEAT
    inc(w);
    s[w] := HEXCONV[BYTE(zahl) AND $F];
    zahl := zahl shr 4;
  UNTIL zahl=0;
  hex[0]:=char(w);
  wx:=1;
  FOR w:= w DOWNTO 1 DO BEGIN
    hex[w]:=s[wx];
    inc(wx);
  END;
END;


FUNCTION hexF(zahl :LONGINT; Stellen : INTEGER) : STRING;
  {wandelt eine beliebige LONGINT-Zahl in einen Hexadezimal-String,
   Anzahl der Hex-Stellen ist definierbar}
VAR
    ziffer : Byte;
    w      : WORD;
    l_zahl: longint;
const
	HEXCONV : array [0..$F] of char = ('0','1','2','3','4','5','6','7',
					   '8','9','A','B','C','D','E','F');
BEGIN
  hexf[0]:=char(stellen); l_zahl:=zahl;
  FOR w:=stellen downto 1 do BEGIN
    hexf[w] := HEXCONV[BYTE(l_zahl) AND $F];
    l_zahl := l_zahl shr 4;
  END;
  if l_zahl<>0 then BEGIN
    hexf:=hex(zahl);
  END;
END;


(**************************************************************************)
(* Konvertierungsroutine fr binre Ausgabe *******************************)
(**************************************************************************)
FUNCTION bit(b:byte) : string; {Hilfsfunktion fr binre Ausgabe}
var j,i : integer;
    s : string;
BEGIN
  s[0]:=#8; i:=b;
  FOR j:=1 to 8 do BEGIN
    if i and 128 > 0 then s[j]:='1' else s[j]:='0';
    i:=i and $FF;
    i:=i shl 1;
  END;
  bit:=s;
END;


END.