{****************************************************************************
*                             SENDRCV.PAS
*              demonstrates serial communication on a MODULAR-4/486
*	       board using CQ8; operating system MS-DOS
*
*              version     : 1.G.
*	       last change : 15.10.1998
*
*              version     : 1.F.
*	       last change : 26.06.1998
*                            Fehler in init_cq8_parameters beseitigt:
*                            Parameter mssen ab Parameter 8 gelesen werden!!
*              version     : 1.D.
*	       last change : 29.03.1996
*
*              version     : 1.C.
*	       creation    : 08.12.1995
*	       last change : 08.12.1995
*	       author	   : H. Klinkenborg
*
*    (c) by SORCUS Computer GmbH Heidelberg 1995
****************************************************************************}

{****************************************************************************
*  Description: This program sends a string of characters via the serial
*	 interface A of the MODULAR-4/486 board and receives this string
*	 via the serial interface B of the MODULAR-4/486 board. The CQ8-
*      	 programs M7P0500.LIB and M7P0520.LIB have to be in the subdirectory
*        SORCUS\CQ8, this example has to be in subdirectory
*        SORCUS\CQ8\BEISP\PASCAL.
*
*  Compiler: Borland PASCAL 7.0
*
*  hardware requirements: PC with monitor, MODULAR-4/486 board
*
*  software requirements: This program requires the library ml7BIB.TPU
*        and the header ml7BIB.PAS. The pathes of M7P0500.LIB and M7P0520.LIB
*	 are given by constant strings M7P0500 and M7P0520 in SENDRCV.PAS.
* 	 The operating system OSX on the MODULAR-4/486 board should be
*	 ml7-2F.05x or greater
****************************************************************************}

{****************************************************************************
*  history:
*        1.G. 15.10.1998
*
*        1.F. 26.06.1998
*              - Fehler in init_cq8_parameters beseitigt:
*              - Parameter mssen ab Parameter 8 gelesen werden!!
*        1.D. 20.03.1996 08:30; HK
*              - parameter clock_send for transmit and clock_rcv for
*                receive added, both have to be "1"
*                (necessary if using M-COM-2)
*        1.C. 08.12.1995 13:46; HK
*	       - creation of the file
****************************************************************************}

program sendrcv;
uses
    Dos,crt,                           { Include default units }
    ml7bib,ml7error;                   { Include the default library for MODULAR-4/486 and
                                         the unit with the errorhandling routine }

TYPE
    statustype = record
                   status: word;
                   number: longint;
                   end;
CONST
     os_string: string = 'ROM';
     IMtask = $300;                    { No. of interrupt manager task for the serial interfaces
                                         A and B of the MODULAR-4/486 board }
     interrupt_no: word = $90;         { No. of interrupt (MODULAR-4/486
                                         SCC = 90H, modules = 79H,7BH .. 7FH }

     { Path and name of interrupt manager program file }

     M7P0500: string = '..\..\M7P0500.LIB';

     { Path and name of basic communication program }

     M7P0520: string = '..\..\M7P0520.LIB';
     transmit_ctrl_function: word = 5; { Function for transmit control }
     transmit_status_function: word = 6; { Function for transmit status}
     send_string_function: word = 7;   { Function for transmit strings }
     receive_ctrl_function: word = 15; { Function for receive control }
     receive_status_function: word = 16; { Function for receive status}
     receive_string_function:word = 17;{ Function for receive strings }
     rcv_status: statustype = (status:0;number:0);
     tmt_status: statustype = (status:0;number:0);
     error: byte = 0;	               { Error byte }
     ml7_adr: word = $380;	       { Base address of MODULAR-4/486 }
     ml7_err: word = 0;                { Error byte }
     M7P0520_PAR : record
        IMpgm_no: word;                { Pgm-no. of interrupt manager }
        IMtask_no: word;               { Task-no. of interrupt manager }
        slot_no : byte;                { Slot-no. }
        channel: byte;                 { Channel-no. }
        d1: array[1..8] of byte;       { Special Parameters }
        clock_send: byte;              { Clock source for transmit }
	clock_rcv: byte;               { Clock source for receive }
	quartz: longint;	       { Quartz frequency }
	baudrate: longint;	       { Baud rate }
	len: byte;		       { Bits per character }
	start_stop: byte;              { Start-/Stop-Bits }
	parity: byte;                  { Parity }
        end = (IMpgm_no:$500;IMtask_no:IMtask;slot_no:0;channel:0;
                                d1:(0,0,0,0,0,0,0,0);clock_send:1;clock_rcv:1;
                                quartz:$737;baudrate:9600;len:8;start_stop:1;
                                parity:0);
    index: integer= 0;                 { Index of in_data_str }
    first: integer = 0;                { Flag }

var
   out_data_str: string;               { Output String }
   in_data_str1: string;               { Input String 1 }
   in_data_str2: string;               { Input String 2 }

procedure check_func_error(slot_no: byte; channel: byte;
                                            error: byte; func: word); forward;

{****************************************************************************
*  init_cq8
*     This procedure resets the MODULAR-4/486 board and transfers and in-
*     stalls the communication programs M7P0500.LIB and M7P0520.LIB
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for installation of communication
*		     program M7P0520.LIB (0 = interface A, 1 = interface B)
*
*     first:	 Q    flag used for reset of MODULAR-4
*		     = 0: ml7BIB is initalized, MODULAR-4 board is reset
*			  and macros are made interruptible, interrupt
*			  manager program M7P0500.LIB for both serial
*			  interfaces on MODULAR-4 board is transferred and
*		       	  installed
*		     = 1: communication program M7P0520.LIB for channel is
*			  transferred and installed
*
****************************************************************************}

procedure init_cq8 (slot_no: byte; channel: byte);
var
   task: word;

begin
   if (first = 0) then begin
      ml7_bib_startup(LANGUAGE_PASCAL);{ Init library ml7BIB }

      { Reset of MODULAR-4/486 board and loading OSX }

      ml7_err := ml7_reset(0,ml7_adr,0,0,0,@error_handler,os_string,10);

      if (ml7_err <> 0) then begin     { Check error }
       	 writeln('Error during reset of MODULAR-4/486');
	 ml7_exit;                     { Close ml7BIB }
	 halt(0);                      { Stop program }
      end;

      { Set macros interruptible }

      ml7_set_macro_interruptible(TRUE);

      { Transfer and install communication program for serial interfaces
	on the MODULAR-4 board }

      ml7_err := ml7_transfer_and_install(M7P0500,0,IMtask,$500,
							interrupt_no,$809,0);

      if (ml7_err <> 0) then begin     { Check error }
         writeln('Error during transfer and install of interrupt ',
							 'manager program');
	 ml7_exit;                     { Close ml7BIB }
	 halt(0);                      { Stop program }
      end;

      first := 1;                      { Set flag to 1 }
   end;
   { Transfer and install communication program for serial interface, which
     is defined in channel }

   task := $310 + $08 * slot_no + channel;

   ml7_err := ml7_transfer_and_install(M7P0520,0,task,$520,0,$808,0);

   if (ml7_err <> 0) then begin        { Check error }
      writeln('Error during transfer and install of communication program');
      ml7_exit;                        { Close ml7BIB }
      halt(0);                         { Stop program }
   end;
end;

{****************************************************************************
*  init_cq8_parameters
*     This procedure sets some parameters of the task M7P0520.LIB
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*                    (0 = interface A, 1 = interface B)
*
*     quartz:        frequency of quartz on MODULAR-4 board, default
*		     7.3728 MHz
*
*     baudrate:	     baud rate, e.g. 9600 baud
*
*     len:	     bits per character, e.g. 8 bits
*
*     start_stop:    start and stop bits, e.g. 1 stop bit
*
*     parity:	     parity, e.g. no parity = 0
*
****************************************************************************}

procedure init_cq8_parameters(slot_no: byte; channel: byte; quartz: longint;
			 baudrate: longint; len: byte;
			 start_stop: byte; parity: byte);
var
   task: word;			       { Number of communication task }
   i : integer;
const
   out_data_word: word = 3;	       { Prepare send or receive and clear
					  buffer }
   in_data_word: word = 0;             { No parameters to be expected }

   outsize: word = 2;		       { Two bytes to transfer }

   insize: word = 0;                   { No parameters to be returned }

begin
   { Read parameters from task M7P0520 of defined channel into record
     M7P0520_PAR, starting with slot_no.
     The Program and task number are predefined in record M7P0520_PAR,
     d1 is predefined in M7P0520.LIB, both may not be changed}

   task := $310 + $08 * slot_no + channel;

   ml7_read_par_block(task,8,sizeof(M7P0520_PAR)-4,M7P0520_PAR.slot_no);

   { Set parameters channel, clock_send, clock_rcv, quartz, baudrate, len,
     start_stop and parity in record M7P0520_PAR }

   M7P0520_PAR.slot_no := slot_no;     { No. of slot, where module is
					  inserted (0 = base board) }
   M7P0520_PAR.channel := channel;     { No. of serial interface }
   M7P0520_PAR.clock_send := 1;        { Clock = baudrate generator }
   M7P0520_PAR.clock_rcv := 1;         { Clock = baudrate generator }
   M7P0520_PAR.quartz := quartz;       { Quartz-frequency }
   M7P0520_PAR.baudrate := baudrate;   { Baud rate }
   M7P0520_PAR.len := len;	       { Bits/character }

   { No. of start/stop bits }

   M7P0520_PAR.start_stop := start_stop;
   M7P0520_PAR.parity := parity;       { Parity }

   { Write parameters from record M7P0520_PAR to the task M7P0520 of defined
     channel }

   ml7_write_par_block(task,4,sizeof(M7P0520_PAR),M7P0520_PAR);

   { Activate task of defined channel }

   ml7_call_proc(task,2);

   { Install transmit control of serial interface defined in channel }

   ml7_call_func(task,transmit_ctrl_function,outsize,
			    out_data_word,0,insize,in_data_word,error);

   { Check, whether an error occurs during function call }

   check_func_error(slot_no,channel,error,transmit_ctrl_function);

   { Install receive control of serial interface defined in channel }

   ml7_call_func(task,receive_ctrl_function,outsize,
			    out_data_word,0,insize,in_data_word,error);

   { Check, whether an error occurs during function call }

   check_func_error(slot_no,channel,error,receive_ctrl_function);
end;

{****************************************************************************
*  send_string
*     This function sends a string via the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     s:             string to be send
*
*     return code:   error code from function call
*
****************************************************************************}

function send_string (slot_no: byte; channel: byte; var s:string): byte;
var
   dummy_insize: word;                 { No. of byte to be returned }
   dummy_in_data_str:string;	       { Dummy string for returned byte }
   error: byte;			       { Error byte of function call }
   task:word;			       { Number of communication task }

begin
   { Calculate task number from slot_no and channel }

   task := $310 + $08 * slot_no + channel;

   { Send string via serial interface of MODULAR-4/486 given in channel }

   ml7_call_func(task,send_string_function,length(s),s[1],0,dummy_insize,
						    dummy_in_data_str,error);

   send_string := error;       	       { Return error code of function call }
end;


{****************************************************************************
*  send_status
*     This procedure returns the transmit status of the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     status:        status of transmitter
*
*
****************************************************************************}
function send_status (slot_no: byte; channel: byte; var status: statustype): byte;
var
   dummy_out_data_str:string;	       { Dummy string }
   insize: word;
   task:word;			       { Number of communication task }

begin
   { Calculate task number from slot_no and channel }

   task := $310 + $08 * slot_no + channel;

   { Send string via serial interface of MODULAR-4/486 given in channel }

   ml7_call_func(task,transmit_status_function,0,dummy_out_data_str,6,insize,status,error);

   send_status := error;
end;

{****************************************************************************
*  rcv_string
*     This function receives a string via the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     s:             string to be received
*
****************************************************************************}

function rcv_string (slot_no: byte; channel: byte; var s: string): byte;
const
   maxinsize: word = 255;              { Maximum number of byte to be
   				         received }
var
   dummy_out_data_str: string;         { Dummy transfer string }
   insize: word;                       { Number of bytes received }
   error: byte;                        { Error byte of function call }
   task: word;                         { Number of communication task }

begin

   { Calculate task number from slot_no and channel }

   task := $310 + $08 * slot_no + channel;

   { Receive string via serial interface of MODULAR-4/486 defined in channel }

   ml7_call_func(task,receive_string_function,0,dummy_out_data_str,
						  maxinsize,insize,s[1],error);
   s[0] := chr(insize);		       { Set string size }
   rcv_string := error;                { Return error code of function call }
end;

{****************************************************************************
*  receive_status
*     This procedure returns the receiver status of the given channel
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:       channel number for communication program M7P0520.LIB
*		     (0 = interface A, 1 = interface B)
*
*     status:        status of receiver
*
*
****************************************************************************}
function receive_status (slot_no: byte; channel: byte; var status: statustype): byte;
var
   dummy_out_data_str:string;	       { Dummy string }
   insize: word;
   task:word;			       { Number of communication task }

begin
   { Calculate task number from slot_no and channel }

   task := $310 + $08 * slot_no + channel;

   { Send string via serial interface of MODULAR-4/486 given in channel }

   ml7_call_func(task,receive_status_function,0,dummy_out_data_str,6,insize,status,error);

   receive_status := error;
end;

{****************************************************************************
*  check_func_error
*     This procedure checks, wether an error occurs during a function call
*
*     slot_no:       slot number of communication module, (= 0: base board)
*
*     channel:	     channel number (0 = interface A, 1 = interface B)
*
*     error:         error number returned by function
*
*     func:          function number of corresponding task
*
****************************************************************************}

procedure check_func_error(slot_no: byte; channel: byte;
                                                     error: byte; func: word);
var
   err_status: byte;                   { Status byte of communication
				         program }
   task: word;			       { No. of communication task }

begin
   if (error <> 0) then begin          { Print error message, if error
				         <> 0 }
      { Calculate task number from slot_no and channel }

      task := $310 + $08 * slot_no + channel;

      { Print error message }
      gotoxy(1,20);
      writeln('Error in function ',func,' of task ',task,': ',error);

      { Read parameter of communication program and print value }

      ml7_read_par_block(task,0,1,err_status);
      writeln('Errorbyte (Parameter 1) = ',err_status);
      ml7_exit;                        { Close ml7BIB }
      halt(0);                         { Stop program }
   end;
end;

{****************************************************************************
*  sendrcv
*     This main routine sends a string and receives it
*
****************************************************************************}

begin

   clrscr;			       { Clear screen }

   { Print title message to screen }

   writeln('Example 1 for CQ8 communication    Version 1.G. 15.10.1998 07:45');

   { Initialize output and input strings }

   out_data_str := 'This is a test of CQ8 communication!';
   in_data_str1 := '';
   in_data_str2 := '';

   init_cq8(M7P0520_PAR.slot_no,0);    { Initialize channel 0 on modul in
				         predefined slot }
   init_cq8(M7P0520_PAR.slot_no,1);    { Initialize channel 1 on modul in
				         predefined slot }

   { Initialize the parameters quartz, baudrate, len, start_stop and parity
     for channel 0 on modul in predefined slot }

   init_cq8_parameters(M7P0520_PAR.slot_no,0,$737,300,8,1,0);

   { Initialize the parameters quartz, baudrate, len, start_stop and parity
     for channel 1 on modul in predefined slot }

   init_cq8_parameters(M7P0520_PAR.slot_no,1,$737,300,8,1,0);

   error := send_string(M7P0520_PAR.slot_no,0, out_data_str);

   { Check, whether an error occurs during function call }

   check_func_error(M7P0520_PAR.slot_no,0,error,send_string_function);

   { Print message to screen }

   gotoxy(1,5);writeln('The ',length(out_data_str):2,' transmitted characters: ',
                                                                out_data_str);

   { repeat receiving string until all characters are received or
     keypressed }

   while not keypressed do
   (*((length(in_data_str2) <> length(out_data_str))
                                                and (not keypressed)) do *)
     begin

      error := send_status(M7P0520_PAR.slot_no,0,tmt_status);
      gotoxy(1,4);write('Sende-Status: ',tmt_status.status,' Anzahl Byte im Puffer: ',tmt_status.number:2);

      { Check, whether an error occurs during function call }

      check_func_error(M7P0520_PAR.slot_no,0,error,transmit_status_function);

      error := receive_status(M7P0520_PAR.slot_no,1,rcv_status);

      gotoxy(1,3);write('Empfangs-Status: ',rcv_status.status,' Anzahl empfangene Byte: ',rcv_status.number:2);

      { Check, whether an error occurs during function call }

      check_func_error(M7P0520_PAR.slot_no,1,error,receive_status_function);

      error := rcv_string(M7P0520_PAR.slot_no,1,in_data_str1);

      { Check, whether an error occurs during function call }

      check_func_error(M7P0520_PAR.slot_no,1,error,receive_string_function);

      { Concat received string with in_data_str2 }

      in_data_str2 := in_data_str2 + in_data_str1;

      gotoxy(1,6);
      writeln('The ',length(in_data_str2):2,' received characters   : ',
					                        in_data_str2);
      if (length(in_data_str2) = length(out_data_str)) then
        begin
          gotoxy(1,5);
          write('                                                                     ');
          gotoxy(1,6);
          write('                                                                     ');
          error := send_string(M7P0520_PAR.slot_no,0, out_data_str);
          gotoxy(1,5);
          writeln('The ',length(out_data_str):2,' transmitted characters: ',
                                                                out_data_str);
          in_data_str1 := '';
          in_data_str2 := '';
        end;
    end;

   ml7_exit;                	       { Close ml7BIB }
end.



